import greenfoot.Actor;
import greenfoot.World;
import greenfoot.GreenfootImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.RenderingHints;

/**
 * Board
 * 
 * @author Taylor Born
 * @version December 2013
 */
public abstract class Board extends Actor
{
    private static final Font AXIS_FONT = new Font("Helvetica", Font.PLAIN, 12);
    public static final int SIZE = 10;
    public static final int CELL_SIZE = Screen.HEIGHT / 2 / SIZE;
    public static final int WIDTH = CELL_SIZE * SIZE;
    private static final int AXIS_OFFSET = (SIZE + 1) * CELL_SIZE / 2;

    private AxisMarkers verticalAxis = new AxisMarkers(true);
    private AxisMarkers horizontalAxis = new AxisMarkers(false);
    protected Attack[][] attacks;

    public Board()
    {
        GreenfootImage image = new GreenfootImage(SIZE * CELL_SIZE, SIZE * CELL_SIZE);
        image.setColor(Color.BLUE);
        image.fill();
        image.setColor(new Color(0, 80, 255));
        for (int i = 1; i < SIZE; i++)
        {
            int n = i * CELL_SIZE;
            image.drawLine(0, n, image.getWidth(), n);
            image.drawLine(n, 0, n, image.getHeight());
        }
        setImage(image);
    }
    
    public void reset()
    {
        attacks = new Attack[SIZE][SIZE];
        getWorld().removeObjects(getWorld().getObjects(Attack.class));
    }
    
    protected void newAttack(int column, int row, boolean hit)
    {
        attacks[row][column] = new Attack(hit);
        addInWorldAt(attacks[row][column], column, row);
    }
    
    protected void addInWorldAt(Actor actor, int column, int row)
    {
        getWorld().addObject(actor, getXFor(column), getYFor(row));
    }
    protected int getXFor(int column)
    {
        return getX() - WIDTH / 2 + column * CELL_SIZE + CELL_SIZE / 2;
    }
    protected int getYFor(int row)
    {
        return getY() - WIDTH / 2 + row * CELL_SIZE + CELL_SIZE / 2;
    }
    
    @Override
    public void addedToWorld(World world)
    {
        world.addObject(verticalAxis, getX() - AXIS_OFFSET, getY());
        world.addObject(horizontalAxis, getX(), getY() - AXIS_OFFSET);
    }
    
    @Override
    public void setLocation(int x, int y)
    {
        super.setLocation(x, y);
        verticalAxis.setLocation(x - AXIS_OFFSET, y);
        horizontalAxis.setLocation(x, y - AXIS_OFFSET);
    }
    
    private class AxisMarkers extends Actor
    {
        public AxisMarkers(boolean vertical)
        {
            GreenfootImage image = new GreenfootImage((vertical ? 1 : SIZE) * CELL_SIZE, (vertical ? SIZE : 1) * CELL_SIZE);
            Graphics2D g = image.getAwtImage().createGraphics();
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g.setFont(AXIS_FONT);
            g.setColor(Color.BLACK);
            for (int i = 0; i < SIZE; i++)
                g.drawString(vertical ? Character.toString((char)('A' + i)) : Integer.toString(i + 1), vertical ? 2 : 2 + CELL_SIZE * i, vertical ? CELL_SIZE * (i + 1) - 4 : CELL_SIZE - 2);
            g.dispose();
            setImage(image);
        }
    }
    
    private class Attack extends Actor
    {
        public Attack(boolean hit)
        {
            GreenfootImage image = new GreenfootImage(Board.CELL_SIZE / 2, Board.CELL_SIZE / 2);
            image.setColor(hit ? Color.RED : Color.WHITE);
            image.fillOval(0, 0, image.getWidth() - 1, image.getHeight() - 1);
            setImage(image);
        }
    }
}