import greenfoot.UserInfo;
import greenfoot.Greenfoot;
import greenfoot.MouseInfo;
import greenfoot.GreenfootImage;
import greenfoot.World;
import java.util.ArrayList;
import java.awt.Color;
import java.awt.Point;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.RenderingHints;

/**
 * ContactsBox
 * 
 * @author Taylor Born
 * @version April - December 2013
 */
public class ContactsBox extends WindowComponent
{
    private static final Font FONT = new Font("Helvetica", Font.PLAIN, 12);
    
    // Size of this ListBox
    private Point size = new Point(650, 300);
    private GreenfootImage pic = new GreenfootImage((int)size.getX(), (int)size.getY());
    
    // Height of scrollBar
    private int scrollBar = 40;
    
    // Y position of where mouse pressed on scrollBar, used as reference for when dragged
    private int mouseY;
    
    private final int ENTRY_HEIGHT = 22;
    
    private volatile ArrayList<UserInfoStorage> storage = new ArrayList<UserInfoStorage>();
//     private TextBox searchBox = new TextBox(new Point(130, 16), "");
    private volatile ArrayList<UserInfoStorage> storagMain = new ArrayList<UserInfoStorage>();
    
    private ScrollingListener scroller = initializeScroller();
    private int scroll;
    private ArrayList<Integer> index = new ArrayList<Integer>();
    private boolean allowMultipleSelect = false;
    private boolean dragging;
    private Point lastMouse = new Point(-25, -25);
    
    private Color backColor = Color.WHITE;
    private Color textColor = Color.BLACK;
    private Color hoverColor = new Color(185, 211, 225);
    private Color selectColor = Color.YELLOW;
    private Color scrollColor = Color.RED;
    
    public static final int SORT_BY_NAME        = 0;
    public static final int SORT_BY_GAME_STATUS = 1;
    public static final int SORT_BY_TIME_STATUS = 2;
    private int sortCriteria = SORT_BY_NAME;
    public void sortBy(int criteria)
    {
        sortCriteria = criteria;
        while (!requestToUseStorage())
        {}
        
        boolean swap = true;
        while (swap)
        {
            swap = false;
            for (int i = 1; i < storagMain.size(); i++)
                if (storagMain.get(i).compareTo(storagMain.get(i - 1)) < 0)
                {
                    UserInfoStorage temp = storagMain.get(i);
                    storagMain.set(i, storagMain.get(i - 1));
                    storagMain.set(i - 1, temp);
                    swap = true;
                }
        }
        
        usingStorage = false;
        refilterSearch();
    }

    public ContactsBox()
    {
//         searchBox.setMessage("Search...");
        setImage(pic);
    }
    
    private volatile boolean usingStorage = false;
    private synchronized boolean requestToUseStorage()
    {
        if (usingStorage)
            return false;
        usingStorage = true;
        return true;
    }
    
    public void add(UserInfo user)
    {
        while (!requestToUseStorage())
        {}
        UserInfoStorage uis = new UserInfoStorage(user);
        boolean inserted = false;
        for (int i = 0; i < storagMain.size(); i++)
            if (uis.compareTo(storagMain.get(i)) < 0)
            {
                storagMain.add(i, uis);
                inserted = true;
                break;
            }
        if (!inserted)
            storagMain.add(uis);
        usingStorage = false;
        refilterSearch();
    }
    public void addedToWorld(World world)
    {
        super.addedToWorld(world);
        Greenfoot.getKey();
//         world.addObject(searchBox, getX(), getY() + (int)size.getY() / 2 + 18);
    }
    public void removeFromWorld()
    {
        super.removeFromWorld();
        deselect();
//         searchBox.removeFromWorld();
    }
    
    /**
     * Update the image.
     */
    private void draw()
    {
        pic.clear();
        pic.setColor(backColor);
        pic.fill();
        
        while (!requestToUseStorage())
        {}
        
        int listSize = storage.size();
        
        // Draw items.
        for (int i = scroll / ENTRY_HEIGHT; i < listSize && i - scroll / ENTRY_HEIGHT < size.getY() / ENTRY_HEIGHT + 1; i++)
        {
            int y = ENTRY_HEIGHT * i - scroll + ENTRY_HEIGHT / 2;
            
            // If selected.
            if (index.contains(i))
            {
                pic.setColor(selectColor);
                pic.fillRect(0, ENTRY_HEIGHT * i - scroll, (int)size.getX() - 10, ENTRY_HEIGHT);
            }
            // If hovered over.
            else if (mouseOverThis() && !dragging && lastMouse.getX() < getX() + size.getX() / 2 - 10 && lastMouse.getY() > getY() - size.getY() / 2 + ENTRY_HEIGHT * i - scroll && lastMouse.getY() <= getY() - size.getY() / 2 + ENTRY_HEIGHT * i - scroll + ENTRY_HEIGHT)
            {
                pic.setColor(hoverColor);
                pic.fillRect(0, ENTRY_HEIGHT * i - scroll, (int)size.getX() - 10, ENTRY_HEIGHT);
            }
            
            GreenfootImage userImage = storage.get(i).getUserImage();
            pic.drawImage(userImage, 2, y - userImage.getHeight() / 2);
            
            Graphics2D g = pic.getAwtImage().createGraphics();
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g.setFont(FONT);
            g.setColor(textColor);
            g.drawString(storage.get(i).getUserName(), 2 + userImage.getWidth() + 6, y + 6);
            
            OnlinePresence op = UserOnlineStatusTracker.getOnlinePresenceFor(storage.get(i).getUserName());
            g.drawString(op.getGameStatus() + "", (int)size.getX() / 2, y + 6);
            String activityText = op.toString();
            g.drawString(activityText, (int)size.getX() - 10 - g.getFontMetrics().charsWidth(activityText.toCharArray(), 0, activityText.length()), y + 6);
            
            g.dispose();
        }
        
        // Scroll bar.
        int n = scroller.getScroll();
        if (listSize > size.getY() / ENTRY_HEIGHT)
        {
            if (mouseOverThis() && getWorld() != null)
            {
                scroll += n;
                if (scroll < 0)
                    scroll = 0;
                else if (scroll + size.getY() > listSize * ENTRY_HEIGHT)
                    scroll = listSize * ENTRY_HEIGHT - (int)size.getY();
            }
            pic.setColor(scrollColor);
            pic.fillRect((int)size.getX() - 9, (int)(scroll / (double)(listSize * ENTRY_HEIGHT - size.getY()) * (size.getY() - scrollBar)), 9, scrollBar);
        }
        
        usingStorage = false;
    }
    
    private void refilterSearch()
    {
        while (!requestToUseStorage())
        {}
        deselect();
        scroll = 0;
        storage.clear();
//         int length = searchBox.getText().length();
        for (UserInfoStorage uis : storagMain)
//             if (uis.getUserName().length() >= length && uis.getUserName().substring(0, length).compareToIgnoreCase(searchBox.getText()) == 0)
                storage.add(uis);
        usingStorage = false;
    }
    
    /**
     * Act.
     */
    public void act()
    {
        super.act();
        
//         if (searchBox.hasChanged())
//             refilterSearch();
        
        int listSize = storage.size();
        
        int topOfScrollBar = (int)(scroll / (double)(listSize * ENTRY_HEIGHT - size.getY()) * (size.getY() - scrollBar));
        
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (listSize > size.getY() / ENTRY_HEIGHT && Greenfoot.mousePressed(this) && mouse.getX() - (getX() - size.getX() / 2) > size.getX() - 10 && mouse.getX() - (getX() - size.getX() / 2) < size.getX() && mouse.getY() - (getY() - size.getY() / 2) > topOfScrollBar && mouse.getY() - (getY() - size.getY() / 2) < topOfScrollBar + scrollBar)
        {
            dragging = true;
            mouseY = (mouse.getY() - (getY() - (int)size.getY() / 2)) - topOfScrollBar;
        }
        if (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null))
            dragging = false;
        if (dragging && Greenfoot.mouseDragged(null))
        {
            topOfScrollBar = (mouse.getY() - (getY() - (int)size.getY() / 2)) - mouseY;
            if (topOfScrollBar < 0)
                topOfScrollBar = 0;
            else if (mouse.getY() - (getY() - (int)size.getY() / 2) + (scrollBar - mouseY) > size.getY())
                topOfScrollBar = (int)size.getY() - scrollBar;
            scroll = (int)(topOfScrollBar / (double)(size.getY() - scrollBar) * (listSize * ENTRY_HEIGHT - size.getY()));
        }
        
        if (Greenfoot.mouseDragged(null) || Greenfoot.mouseMoved(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        
        if (Greenfoot.mousePressed(this) && mouse.getX() - (getX() - size.getX() / 2) < size.getX() - 10)// && mouse.getX() - (getX() - size.getX() / 2) > 0 && mouse.getY() - (getY() - size.getY() / 2) > 0 && mouse.getY() - (getY() - size.getY() / 2) < size.getY())
        {
            for (int i = 0; i < listSize; i++)
                if (mouse.getY() - (getY() - size.getY() / 2) > ENTRY_HEIGHT * i - scroll && mouse.getY() - (getY() - size.getY() / 2) <= ENTRY_HEIGHT * i - scroll + ENTRY_HEIGHT)
                {
                    if (Greenfoot.isKeyDown("shift") && allowMultipleSelect)
                    {
                        if (index.contains(i))
                            index.remove(new Integer(i));
                        else
                            index.add(i);
                    }
                    else
                    {
                        index = new ArrayList<Integer>();
                        index.add(i);
                    }
                    setIndexInView(i);
                }
        }
        draw();
    }
    
    /**
     * Ajust scroll amount to set the Object determined by the given index, to be in view.
     * @param index Index for Object to be set to be in view.
     */
    private void setIndexInView(int index)
    {
        if (index > -1 && index < storage.size())
            if (index * ENTRY_HEIGHT < scroll)
                scroll = index * ENTRY_HEIGHT;
            else if ((index + 1) * ENTRY_HEIGHT > scroll + size.getY())
                scroll = (index + 1) * ENTRY_HEIGHT - (int)size.getY();
    }
    
    /**
     * Deselect selection.
     */
    public void deselect()
    {
        index.clear();
    }
    
    /**
     * Determine if there are any Objects selected.
     * @return Whether there are any Objects selected.
     */
    public boolean hasSelection()
    {
        return index.size() > 0;
    }
    
    /**
     * Get the selected Object. Meant to be used when multiple selecting is disabled, otherwise getSelection() is more appropriate.
     * @return The selected Object. Null if none selected.
     * @see getSelection()
     */
    public UserInfo getOneSelection()
    {
        if (index.size() == 0)
            return null;
        return storage.get(index.get(0)).getUser();
    }
    
    public String getSelectedName()
    {
        return storage.get(index.get(0)).getUserName();
    }
    public int getSelectedScore()
    {
        return storage.get(index.get(0)).getScore();
    }
    
    public ArrayList<UserInfoStorage> getUserInfoStorageList()
    {
        return storagMain;
    }
    
    public class UserInfoStorage implements Comparable<UserInfoStorage>
    {
        private String userName;
//         private GreenfootImage userImage;
        private UserInfo user;
        
        private int score;
//         private String[] strings = new String[UserInfo.NUM_STRINGS];
//         private int[] ints = new int[UserInfo.NUM_INTS];
        
        public UserInfoStorage(UserInfo user)//, int rank)
        {
            this.user = user;
            score = user.getScore();
            
            userName = user.getUserName();
            
//             userImage = UserIcons.commit(userName, user);
            
            
//             for (int i = 0; i < UserInfo.NUM_STRINGS; i++)
//                 strings[i] = user.getString(i);
//             for (int i = 0; i < UserInfo.NUM_INTS; i++)
//                 ints[i] = user.getInt(i);
        }
//         public UserInfoStorage(UserInfo user)
//         {
//             this(user, user.getRank());
//         }
        public String getUserName()
        {
            return userName;
        }
        public GreenfootImage getUserImage()
        {
            return UserIcons.getIconFor(userName);
        }
        public UserInfo getUser()
        {
            return user;
        }
//         public int getRank()
//         {
//             return rank;
//         }
        public int getScore()
        {
            return score;
        }
        
        @Override
        public int compareTo(UserInfoStorage other)
        {
            if (sortCriteria == SORT_BY_NAME)
                return userName.compareToIgnoreCase(other.getUserName());
            
            OnlinePresence otherOP = UserOnlineStatusTracker.getOnlinePresenceFor(other.getUserName());
            OnlinePresence myOP = UserOnlineStatusTracker.getOnlinePresenceFor(userName);
            
            if (sortCriteria == SORT_BY_GAME_STATUS)
            {
                String order = "R-G";
                int comparison = (new Integer(order.indexOf(myOP.getGameStatus()))).compareTo(new Integer(order.indexOf(otherOP.getGameStatus())));
                if (comparison != 0) // Otherwise will compare by last activity
                    return comparison;
            }
            
            int otherSeconds = otherOP.secondsAgoOfLastActivity();
            int mySeconds = myOP.secondsAgoOfLastActivity();
            if (otherSeconds == -1 && mySeconds == -1)
                return 0;
                
            if (otherSeconds == -1)
                return -1;
            if (mySeconds == -1)
                return 1;
            return (new Integer(mySeconds)).compareTo(new Integer(otherSeconds));
        }
    }
}