import greenfoot.World;
import greenfoot.Actor;
import greenfoot.GreenfootImage;
import greenfoot.UserInfo;
import java.awt.Point;
import java.awt.Font;
import java.awt.Color;

/**
 * CorrespondentScreen
 * 
 * @author Taylor Born
 * @version December 2013
 */
public class CorrespondentScreen extends World
{
    private static final int WIDTH = Screen.WIDTH;
    private static final int HEIGHT = Screen.HEIGHT;
    public static CorrespondentScreen main;

    private volatile ContactsBox contacts = new ContactsBox();
    private static final int BTN_HEIGHT = 24;
    private Button sortByActivityBtn = new Button("Last Known Activity", new Point(120, BTN_HEIGHT));
    private Button sortByGameBtn = new Button("Requesting you / in-Game", new Point(150, BTN_HEIGHT));
    private Button sortByNameBtn = new Button("User", new Point(40, BTN_HEIGHT));

    public CorrespondentScreen()
    {
        super(WIDTH, HEIGHT, 1);
        main = this;
        
        if (UserInfo.isStorageAvailable())
        {
            // Lock out others when testing
//             String userName = UserInfo.getMyInfo().getUserName();
//             if (!userName.equals("bourne") && !userName.equals("PrivateKey"))
//                 return;
            
            CodeKeyUser.enter();
            if (UserInfo.getMyInfo().getUserName().equals("PrivateKey"))
                return;
            
            new Screen();
            
            addObject(contacts, WIDTH / 2, HEIGHT / 2);
//             Label lbl = new Label("Last Known Activity:");
            
            int btnY = HEIGHT / 2 - contacts.getGUIHeight() / 2 - BTN_HEIGHT / 2 - 4;
            addObject(sortByNameBtn, WIDTH / 2 - contacts.getGUIWidth() / 2 + sortByNameBtn.getGUIWidth() / 2 + 5, btnY);
            addObject(sortByGameBtn, WIDTH / 2, btnY);
            addObject(sortByActivityBtn, WIDTH / 2 + contacts.getGUIWidth() / 2 - sortByActivityBtn.getGUIWidth() / 2 - 15, btnY);
        }
        else
            // Notify user to log into site
            addObject(new Actor(){{ setImage(new GreenfootImage("Please login to site.\nUserInfo required.", 30, Color.BLACK, Color.WHITE)); }}, WIDTH / 2, HEIGHT / 2);
    }
    
    public void newUser(UserInfo user)
    {
        contacts.add(user);
    }
    
    @Override
    public void act()
    {
        if (contacts.hasSelection())
        {
            Screen.main.newGame(contacts.getOneSelection(), contacts.getSelectedName(), contacts.getSelectedScore());
            contacts.deselect();
        }
        else
        {
            if (sortByNameBtn.wasClicked())
                contacts.sortBy(ContactsBox.SORT_BY_NAME);
            else if (sortByGameBtn.wasClicked())
                contacts.sortBy(ContactsBox.SORT_BY_GAME_STATUS);
            else if (sortByActivityBtn.wasClicked())
                contacts.sortBy(ContactsBox.SORT_BY_TIME_STATUS);
        }
    }
    
    @Override
    public void started()
    {
        Screen.running = true;
    }
    
    @Override
    public void stopped()
    {
        Screen.running = false;
    }
}