import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;
import greenfoot.MouseInfo;
import greenfoot.World;
import greenfoot.Actor;
import java.awt.Color;
import java.awt.Point;

/**
 * EnemyBoard
 * 
 * @author Taylor Born
 * @version December 2013
 */
public class EnemyBoard extends Board
{
    private boolean selecting;
    private Point selected;
    private SelectingIndicator indicator = new SelectingIndicator();
    
    @Override
    public void act()
    {
        if (!selecting)
            return;
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseMoved(this))
        {
            int xOnThis = mouse.getX() - (getX() - WIDTH / 2);
            int yOnThis = mouse.getY() - (getY() - WIDTH / 2);
            
            int column = xOnThis / CELL_SIZE;
            int row = yOnThis / CELL_SIZE;
            if (attacks[row][column] == null)
                indicator.set(column, row);
            else
                indicator.removeFromWorld();
        }
        else if (!Greenfoot.mouseMoved(indicator) && Greenfoot.mouseMoved(null))
            indicator.removeFromWorld();
    }
    
    public void revealShips(String report)
    {
        for (int i = 0; i < report.length(); i += 2)
            addInWorldAt(new RevealMarker(), report.charAt(i) - 'A', report.charAt(i + 1) - 'A');
    }
    
    @Override
    public void reset()
    {
        super.reset();
        selecting = false;
        selected = null;
        getWorld().removeObjects(getWorld().getObjects(RevealMarker.class));
    }
    
    public Point getSelection()
    {
        return selected;
    }
    
    public void enableSelecting()
    {
        selecting = true;
        selected = null;
    }
    
    public void disableSelecting()
    {
        selecting = false;
        selected = null;
    }
    
    public void recordAttack(boolean hit)
    {
        newAttack((int)selected.getX(), (int)selected.getY(), hit);
    }
    
    private class SelectingIndicator extends Actor
    {
        private int column, row;
        
        public SelectingIndicator()
        {
            GreenfootImage image = new GreenfootImage(CELL_SIZE, CELL_SIZE);
            image.setColor(Color.RED);
            image.drawRect(0, 0, image.getWidth() - 1, image.getHeight() - 1);
            setImage(image);
        }
        
        @Override
        public void act()
        {
            if (Greenfoot.mousePressed(this))
            {
                selected = new Point(column, row);
                selecting = false;
                removeFromWorld();
            }
        }
        
        public void set(int column, int row)
        {
            this.column = column;
            this.row = row;
            if (getWorld() == null)
                addInWorldAt(this, column, row);
            else
                setLocation(getXFor(column), getYFor(row));
        }
        
        public void removeFromWorld()
        {
            World world = getWorld();
            if (world != null)
                world.removeObject(this);
        }
    }
    
    private class RevealMarker extends Actor
    {
        public RevealMarker()
        {
            GreenfootImage image = new GreenfootImage(Board.CELL_SIZE - 4, Board.CELL_SIZE - 4);
            image.setColor(Color.GREEN);
            image.drawRect(0, 0, image.getWidth() - 1, image.getHeight() - 1);
            setImage(image);
        }
    }
}