import greenfoot.World;
import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;
import greenfoot.UserInfo;
import greenfoot.Actor;
import java.util.List;
import java.awt.Point;
import java.awt.Color;

/**
 * Screen
 * 
 * @author Taylor Born
 * @version December 2013
 */
public class Screen extends World
{
    public static final int WIDTH = 700;
    public static final int HEIGHT = 400;
    public static Screen main;
    public static final int PULL_USERINFO_LIST_SIZE = 31000;
    private static final int TIME_TO_PLACE_SHIPS = 45000;
    private static final int TIME_FOR_TURN = 15000;
    private static final int STATUS_NULL            = 0;
    private static final int STATUS_REQUESTING_GAME = 1;
    private static final int STATUS_GAME_REQUESTED  = 2;
    private static final int STATUS_PLACING_SHIPS   = 3;
    private static final int STATUS_DONE_PLACING    = 4;
    private static final int STATUS_READY_TO_START  = 5;
    private static final int STATUS_MY_TURN         = 6;
    private static final int STATUS_MY_TURN_SENT    = 7;
    private static final int STATUS_OP_TURN         = 8;
    private static final int STATUS_OP_RESULT_SENT  = 9;
    private static final int STATUS_LOST            = 10;
    private static final int STATUS_LOST_WAIT       = 11;
    private static final int STATUS_WON             = 12;
    private static final int STATUS_WON_WAIT        = 13;
    private static final int STATUS_CHAT            = 14;
    
    private static final int CODE_ID = 56;
    
    public static UserInfo myUserInfo;
    public static String myName;
    public static int myScore;
    private int myTurnNumber;
    private volatile boolean missedMyTurn;
    private volatile Point mySelection;
    private static final Point LOST_MY_TURN = new Point(0, 10);
    private int myShipsSunk;
    private long timeStartedPlacingShips;
    private long timeStartedMyTurn;
    private boolean iStart;
    private boolean myGoFirstRoll;
    
    private UserInfo opponentUserInfo;
    private volatile String opponentName;
    private int opponentScore;
    private int opponentLastTurnNumber;
    private int opShipsSunk;
    
    public static volatile boolean running;
    private static UserInfoThread threadA;
    private static UserInfoThread threadB;
    private static UserInfoThread threadC;

    private HomeBoard myBoard = new HomeBoard();
    private EnemyBoard eBoard = new EnemyBoard();
    private ConversationLogView log = new ConversationLogView();
    private Label statusLbl = new Label("");
    private Button cancelRequest = new Button("Cancel", new Point(100, 24));
    private Button donePlacingBtn = new Button("Done Placing Ships", new Point(120, 24));
    private Button exitBtn = new Button("Exit", new Point(100, 24));
    private volatile int status = STATUS_NULL;
    
    private Container container = new Container(new Point(1, 3), 2);
    private ImageHolder opponentImage = new ImageHolder();
    private Label opponentNameLbl = new Label("");
    private Label opponentLastActivityLbl = new Label("");
    
    private Container chatInputContainer = new Container(new Point(1, 2), 0);
    private TextBox input = new TextBox(new Point(200, 35), "");
    private CheckBoxLabel sendOnEnterCheckBox = new CheckBoxLabel("Press Enter to Send", true);
    private Button sendBtn = new Button("Send", new Point(200 - sendOnEnterCheckBox.getGUIWidth() - 10, 23));
    private boolean opponentInChat;

    public Screen()
    {
        super(WIDTH, HEIGHT, 1);
        main = this;
        
        myUserInfo = UserInfo.getMyInfo();
        myName = myUserInfo.getUserName();
        
        myScore = myUserInfo.getScore();
        if (myScore == 0 || myScore == -1)
            initializeScore();
        else
        {
            for (int i = 0; i < 10; i++)
                myUserInfo.setInt(i, 0);
            for (int i = 0; i < 5; i++)
                myUserInfo.setString(i, "");
            myUserInfo.store();
        }
        
        addObject(myBoard, WIDTH - 225 - 225 / 2, HEIGHT / 2);
        addObject(eBoard, WIDTH - 225 / 2, HEIGHT / 2);
        
        input.setMessage("Enter message here...");
        input.setMaxLength(200);
        input.dontAccept("\n");
        sendBtn.setAcceptByEnterKey(true);
        chatInputContainer.addComponent(input);
        Container sendC = new Container(new Point(2, 1));
        sendC.addComponent(sendBtn);
        sendC.addComponent(sendOnEnterCheckBox);
        chatInputContainer.addComponent(sendC);
        
        Container opponentContainer = new Container(new Point(2, 1), 1);
        opponentContainer.addComponent(opponentImage);
        opponentContainer.addComponent(opponentNameLbl);
        
        container.addComponent(opponentContainer);
        container.addComponent(opponentLastActivityLbl);
        container.addComponent(log);
        
        addObject(container, 10 + log.getGUIWidth() / 2, HEIGHT / 2 - 14);
        
        addObject(statusLbl, WIDTH / 2, statusLbl.getGUIHeight() / 2);
        
        if (threadA != null)
        {
            threadA.setToEnd();
            threadB.setToEnd();
            threadC.setToEnd();
        }
        threadA = new MyUserOnlineStatus();
        (new Thread(threadA)).start();
        threadB = new UserOnlineStatusTracker();
        (new Thread(threadB)).start();
        threadC = new OpponentListener();
        (new Thread(threadC)).start();
    }
    
    private void initializeScore()
    {
        while (true)
        {
            // Figure out next available score (ID) and try "claiming" it
            List<UserInfo> userInfos = UserInfo.getTop(PULL_USERINFO_LIST_SIZE);
            myScore = userInfos.size() + 1;
            for (int i = userInfos.size() - 1; i >= 0; i--)
                if (userInfos.get(i).getScore() <= 0)
                    myScore--;
                else
                    break;
            myUserInfo.setInt(9, myScore);
            myUserInfo.setScore(-1); // Indicate I'm trying to initialize my score
            myUserInfo.store();
            
            // Check for colliding score
            userInfos = UserInfo.getTop(PULL_USERINFO_LIST_SIZE);
            int leadingScore = userInfos.get(0).getScore();
            
            if (leadingScore <= 0 || leadingScore == myScore - 1) // Check if leading score one less than my score I'm trying to initialize as
            {
                // If found colliding score claim
                boolean found = false;
                
                // Check other users who are trying to initialize their score
                for (int i = userInfos.size() - 1; i >= 0; i--)
                {
                    UserInfo ui = userInfos.get(i);
                    if (!ui.getUserName().equals(myName))
                    {
                        int sc = ui.getScore();
                        
                        // Check if done checking users trying to initialize their score
                        if (sc >= 0)
                            break;
                        // Check if this user is trying to initialize with the same score
                        if (sc == -1 && ui.getInt(9) == myScore)
                        {
                            // Found collision, so reset
                            myUserInfo.setScore(0);
                            myUserInfo.store();
                            found = true;
                            break;
                        }
                    }
                }
            
                // If no collisions, finalize score
                if (!found)
                {
                    myUserInfo.setScore(myScore);
                    myUserInfo.store();
                    return;
                }
            }
            
            // Failed so try again
            if (Math.random() > .5) // Chance for collisions, someone will get a score finalized
                ThreadUtil.linger(2000);
        }
    }
    public void newGame(UserInfo opponent, String name, int opScore)
    {
        statusLbl.setText("");
        opponentUserInfo = opponent;
        myTurnNumber = 0;
        missedMyTurn = false;
        mySelection = null;
        myShipsSunk = 0;
        
        opponentName = name;
        opponentScore = opScore;
        opponentImage.setImage(UserIcons.getIconFor(name));
        opponentNameLbl.setText(name);
        opponentLastTurnNumber = 0;
        opShipsSunk = 0;
        
        myBoard.reset();
        eBoard.reset();
        status = STATUS_REQUESTING_GAME;
        Greenfoot.setWorld(this);
        log.clear();
        container.setLocation(container.getX(), container.getY());
    }
    
    @Override
    public void act()
    {
        KeysPressed.update();
        
        if (opponentUserInfo != null)
        {
            if (status == STATUS_MY_TURN && mySelection == null) // Order is important
            {
                mySelection = eBoard.getSelection();
                if (mySelection == null && System.currentTimeMillis() - timeStartedMyTurn >= TIME_FOR_TURN)
                {
                    mySelection = LOST_MY_TURN;
                    missedMyTurn = true;
                    eBoard.disableSelecting();
                    record("You missed your turn.");
                }
            }
            
            if (status == STATUS_PLACING_SHIPS)
            {
                int timeSinceStarted = (int)(System.currentTimeMillis() - timeStartedPlacingShips);
                donePlacingBtn.setEnable(timeSinceStarted >= 3000);
                if (donePlacingBtn.wasClicked() || timeSinceStarted >= TIME_TO_PLACE_SHIPS)
                {
                    myBoard.finishPlacing();
                    status = STATUS_DONE_PLACING;
                    removeObject(donePlacingBtn);
                    statusLbl.setText("Waiting on " + opponentName);
                }
                else
                    statusLbl.setText("Place your ships (" + (TIME_TO_PLACE_SHIPS - (System.currentTimeMillis() - timeStartedPlacingShips)) / 1000 + "s remaining)");
            }
            
            if (status == STATUS_CHAT)
            {
                if (sendOnEnterCheckBox.hasChanged())
                {
                    sendBtn.setAcceptByEnterKey(sendOnEnterCheckBox.isChecked());
                    input.dontAccept(sendOnEnterCheckBox.isChecked() ? "\n" : "");
                }
                
                sendBtn.setEnable(!input.getText().trim().isEmpty());
                if (sendBtn.wasClicked() && opponentInChat)
                {
                    log.send(input.getText(), opponentScore);
                    input.clear();
                }
            }
            
            opponentLastActivityLbl.setText("Last Known Activity: " + UserOnlineStatusTracker.getOnlinePresenceFor(opponentName).toString());
        }
        
        if (status != STATUS_CHAT)
        if (exitBtn.wasClicked())
        {
            Greenfoot.setWorld(CorrespondentScreen.main);
            removeObject(exitBtn);
            chatInputContainer.removeFromWorld();
        }
    }
    
    private void record(String text)
    {
        log.addNotification(text);
    }
    
    @Override
    public void started()
    {
        running = true;
    }
    
    @Override
    public void stopped()
    {
        running = false;
        
        removeObject(donePlacingBtn);
        chatInputContainer.removeFromWorld();
        Greenfoot.setWorld(CorrespondentScreen.main);
        status = STATUS_NULL;
        
        while (!UserInfoSemaphore.request())
        {}
        myUserInfo.setInt(1, 0);
        myUserInfo.setInt(2, 0);
        for (int i = 0; i < 5; i++)
            myUserInfo.setString(i, "");
        myUserInfo.store();
        UserInfoSemaphore.release();
    }
    
    private class OpponentListener extends UserInfoThread implements Runnable
    {
        @Override
        public void run()
        {
            while (true)
            {
                while (!running || opponentUserInfo == null)
                    ThreadUtil.linger(2000);
                if (end)
                    return;
                
                while (!UserInfoSemaphore.request())
                {}
                List<UserInfo> userinfos = UserInfo.getTop(PULL_USERINFO_LIST_SIZE);
                UserInfoSemaphore.release();
                
                opponentUserInfo = userinfos.get(userinfos.get(0).getScore() - opponentScore);
                
                OnlinePresence opOP = UserOnlineStatusTracker.getOnlinePresenceFor(opponentName);
                opOP.update(opponentUserInfo.getInt(0));
                
                String opponentInt2Bits = Integer.toBinaryString(opponentUserInfo.getInt(2));
                while (opponentInt2Bits.length() < 32)
                    opponentInt2Bits = '0' + opponentInt2Bits;
                
                boolean opponentReferencingMe = opponentInt2Bits.substring(0, 15).equals(BitStringUtil.numberToBitString(myScore, 15));
                
                if (status > STATUS_GAME_REQUESTED && status < STATUS_LOST)
                    if (opOP.secondsAgoOfLastActivity() >= 20 || !opponentReferencingMe)
                    {
                        showExit();
                        end();
                        statusLbl.setText("Lost connection with " + opponentName + ", game over");
                        eBoard.disableSelecting();
                        removeObject(donePlacingBtn);
//                         UserInfoSemaphore.release();
                        continue;
                    }
                
                String opponentInt1Bits = Integer.toBinaryString(opponentUserInfo.getInt(1));
                while (opponentInt1Bits.length() < 32)
                    opponentInt1Bits = '0' + opponentInt1Bits;
                
                switch (status)
                {
                    case STATUS_REQUESTING_GAME:
                        String myReferenceBits = BitStringUtil.numberToBitString(opponentScore, 15);
                        while (myReferenceBits.length() < 32)
                            myReferenceBits += '0';
                        myUserInfo.setInt(2, BitStringUtil.encodeBitStringTo32BitInt(myReferenceBits));
                        
                        myGoFirstRoll = Math.random() > .5;
                        setMyInt1AndStore("01" + (myGoFirstRoll ? '1' : '0') + "000000" + "0000000" + "0" + "00" + "0000000000000");
                        
                        statusLbl.setText("Requesting game with " + opponentName);
                        addObject(cancelRequest, WIDTH / 2, statusLbl.getGUIHeight() + cancelRequest.getGUIHeight() / 2);
                        status = STATUS_GAME_REQUESTED;
                        break;
                    case STATUS_GAME_REQUESTED:
                        if (cancelRequest.wasClicked())
                        {
                            removeObject(cancelRequest);
                            end();
                            Greenfoot.setWorld(CorrespondentScreen.main);
                            break;
                        }
                        else if ((opponentInt1Bits.substring(0, 2).equals("01") || opponentInt1Bits.substring(0, 2).equals("10")) && opponentReferencingMe)
                        {
                            removeObject(cancelRequest);
                            int compareNames = myName.compareTo(opponentName);
                            iStart = (myGoFirstRoll == (opponentInt1Bits.charAt(2) == '1') ? compareNames < 0 : compareNames > 0);
                            setMyInt1AndStore("10" + (myGoFirstRoll ? '1' : '0') + "000000" + "0000000" + "0" + "00" + "0000000000000");
                            timeStartedPlacingShips = System.currentTimeMillis();
                            statusLbl.setText("Place your ships (" + TIME_TO_PLACE_SHIPS / 1000 + "s remaining)");
                            status = STATUS_PLACING_SHIPS;
                            addObject(donePlacingBtn, WIDTH / 2, HEIGHT - donePlacingBtn.getGUIHeight() / 2);
                        }
                        break;
                    case STATUS_DONE_PLACING:
                        setMyInt1AndStore("11" + "0000000" + "0000000" + (iStart ? "0" : "1") + "00" + "0000000000000");
                        status = STATUS_READY_TO_START;
                        break;
                    case STATUS_READY_TO_START:
                        if (opponentInt1Bits.substring(0, 2).equals("11"))
                            if (iStart)
                                startMyTurn();
                            else
                            {
                                statusLbl.setText(opponentName + "'s turn");
                                status = STATUS_OP_TURN;
                            }
                        break;
                    case STATUS_MY_TURN:
                        if (mySelection != null)
                        {
                            setMyInt1AndStore("11" +
                                              BitStringUtil.numberToBitString(++myTurnNumber, 7) +
                                              BitStringUtil.numberToBitString((int)mySelection.getY() * Board.SIZE + (int)mySelection.getX(), 7) +
                                              "0" + "00" + "0000000000000");
                            
                            // Order is important in relation to the statement:
                            // mySelection = eBoard.getSelection();
                            // In the act method
                            status = STATUS_MY_TURN_SENT;
                            mySelection = null;
                        }
                        else
                            statusLbl.setText("Your Turn (" + (TIME_FOR_TURN - (System.currentTimeMillis() - timeStartedMyTurn)) / 1000 + "s before lose turn)");
                        break;
                    case STATUS_MY_TURN_SENT:
                        if (opponentInt1Bits.charAt(16) == '0')
                        {
                            if (!missedMyTurn)
                            {
                                // Receive response
                                int response = BitStringUtil.bitStringToNumber(opponentInt1Bits.substring(17, 17 + 2));
                                // Update eBoard
                                eBoard.recordAttack(response != 0);
                                Point position = eBoard.getSelection();
                                
                                // Notify me of miss, hit, or hit&sunk
                                record("You attack " + (char)('A' + position.getY()) + "" + (1 + (int)position.getX()) + ", " + (response == 0 ? "miss" : "hit" + (response == 2 ? ", sunk!" : "")));
                                
                                if (response == 2)
                                    if (++myShipsSunk == HomeBoard.MAX_SHIPS)
                                    {
                                        myUserInfo.setString(0, myBoard.getRemainingShipsReport());
                                        setMyInt1AndStore("11" + BitStringUtil.numberToBitString(myTurnNumber, 7) + "0000000" + "1" + "00" + "0000000000000");
                                        status = STATUS_WON;
                                        break;
                                    }
                            }
                            else
                                missedMyTurn = false;
                            
                            statusLbl.setText(opponentName + "'s turn");
                            
                            // Set notification
                            setMyInt1AndStore("11" + BitStringUtil.numberToBitString(myTurnNumber, 7) + "0000000" + "1" + "00" + "0000000000000");
                            status = STATUS_OP_TURN;
                        }
                        break;
                    case STATUS_OP_TURN:
                        int turnNum = BitStringUtil.bitStringToNumber(opponentInt1Bits.substring(2, 2 + 7));
                        if (turnNum != opponentLastTurnNumber)
                        {
                            // Get opponent's attack
                            int coordinate = BitStringUtil.bitStringToNumber(opponentInt1Bits.substring(9, 9 + 7));
                            
                            boolean lost = false;
                            if (coordinate == 100)
                            {
                                // Notify me that opponent missed his turn
                                record(opponentName + " missed their turn.");
                                
                                setMyInt1AndStore("11" + BitStringUtil.numberToBitString(myTurnNumber, 7) + "0000000" + "0" + "00" + "0000000000000");
                            }
                            else
                            {
                                // Update myBoard
                                // Respond, (miss, hit, hit&sunk).
                                // Reset notification
                                int col = coordinate % Board.SIZE;
                                int row = coordinate / Board.SIZE;
                                Boolean result = myBoard.attack(col, row);
                                
                                int r;
                                if (result == null)
                                    r = 0;
                                else if (result)
                                {
                                    r = 2;
                                    if (++opShipsSunk == HomeBoard.MAX_SHIPS)
                                        lost = true;
                                }
                                else
                                    r = 1;
                                
                                record(opponentName + " attacks " + (char)('A' + row) + "" + (1 + col) + ", " + (r == 0 ? "miss" : "hit" + (r == 2 ? ", sunk!" : "")));
                                
                                setMyInt1AndStore("11" + BitStringUtil.numberToBitString(myTurnNumber, 7) + "0000000" + "0" + BitStringUtil.numberToBitString(r, 2) + "0000000000000");
                            }
                            
                            opponentLastTurnNumber = turnNum;
                            status = lost ? STATUS_LOST : STATUS_OP_RESULT_SENT;
                        }
                        break;
                    case STATUS_OP_RESULT_SENT:
                        if (opponentInt1Bits.charAt(16) == '1')
                            startMyTurn();
                        break;
                    case STATUS_LOST:
                        if (opponentInt1Bits.charAt(16) == '1')
                        {
                            eBoard.revealShips(opponentUserInfo.getString(0));
                            statusLbl.setText("You lost to " + opponentName);
                            
                            indicateChat(true);
                            
                            status = STATUS_LOST_WAIT;
                        }
                        break;
                    case STATUS_LOST_WAIT:
                        if (opponentInt2Bits.charAt(15) == '1')
                        {
                            indicateChat(false);
                            
                            startChat();
                        }
                        break;
                    case STATUS_WON:
                        if (opponentInt2Bits.charAt(15) == '1')
                        {
                            statusLbl.setText("You won!");
                            
                            indicateChat(true);
                            
                            status = STATUS_WON_WAIT;
                        }
                        break;
                    case STATUS_WON_WAIT:
                        if (opponentInt2Bits.charAt(15) == '0' || !opponentInt1Bits.substring(0, 2).equals("11") || !opponentReferencingMe)
                        {
                            startChat();
                        }
                        break;
                    case STATUS_CHAT:
                        if (exitBtn.wasClicked())
                        {
                            removeObject(exitBtn);
                            chatInputContainer.removeFromWorld();
                            end();
                            Greenfoot.setWorld(CorrespondentScreen.main);
                        }
                        else if (opponentInChat)
                            if (!opponentInt1Bits.substring(0, 2).equals("11") || !opponentReferencingMe || UserOnlineStatusTracker.getOnlinePresenceFor(opponentName).secondsAgoOfLastActivity() > 15)
                            {
                                record(opponentName + " has left the game session.");
                                opponentInChat = false;
                            }
                            else
                            {
                                String myBits = Integer.toBinaryString(myUserInfo.getInt(2));
                                while (myBits.length() < 32)
                                    myBits = '0' + myBits;
                                
                                boolean needToStore = false;
                                
                                // Check if opponent has message
                                if (opponentInt2Bits.charAt(16) == '1')
                                {
                                    // Check if I have not read message
                                    if (myBits.charAt(17) == '0')
                                    {
                                        String message = "";
                                        for (int i = 1; i <= 4; i++)
                                            message += opponentUserInfo.getString(i);
                                        log.add(opponentName, message);
                                        
                                        // Set read notification
                                        myBits = myBits.substring(0, 17) + '1' + myBits.substring(18);
                                        needToStore = true;
                                    }
                                }
                                // No message, so remove my read notification if it is set
                                else if (myBits.charAt(17) == '1')
                                {
                                    myBits = myBits.substring(0, 17) + '0' + myBits.substring(18);
                                    needToStore = true;
                                }
                                
                                // Check if I sent a message and opponent received it
                                if (myBits.charAt(16) == '1' && opponentInt2Bits.charAt(17) == '1')
                                {
                                    log.messageSent();
                                    
                                    // Remove message
                                    myBits = myBits.substring(0, 16) + '0' + myBits.substring(17);
                                    needToStore = true;
                                }
                                
                                if (needToStore)
                                {
                                    while (!UserInfoSemaphore.request())
                                    {}
                                    myUserInfo.setInt(2, BitStringUtil.encodeBitStringTo32BitInt(myBits));
                                    myUserInfo.store();
                                    UserInfoSemaphore.release();
                                }
                            }
                        break;
                }
            }
        }
        
        private void startChat()
        {
            if (status == STATUS_WON_WAIT && CodeGenerator.haveUser())
                record("Enter this code into bourne's Residents scenario: " + CodeGenerator.getCode(CODE_ID));
            record((status == STATUS_WON_WAIT ? "You won against " : "You lost against ") + opponentName + ". You can now chat.");
            input.clear();
            addObject(chatInputContainer, log.getX(), HEIGHT - (HEIGHT - (log.getY() + log.getGUIHeight() / 2)) / 2);
            opponentInChat = true;
            status = STATUS_CHAT;
            showExit();
        }
        private void setMyInt1AndStore(String bits)
        {
            while (!UserInfoSemaphore.request())
            {}
            myUserInfo.setInt(1, BitStringUtil.encodeBitStringTo32BitInt(bits));
            myUserInfo.store();
            UserInfoSemaphore.release();
        }
        private void indicateChat(boolean notification)
        {
            String bits = BitStringUtil.numberToBitString(opponentScore, 15);
            if (notification)
                bits += '1';
            while (bits.length() < 32)
                bits += '0';
            while (!UserInfoSemaphore.request())
            {}
            myUserInfo.setInt(2, BitStringUtil.encodeBitStringTo32BitInt(bits));
            myUserInfo.store();
            UserInfoSemaphore.release();
        }
        private void showExit()
        {
            addObject(exitBtn, exitBtn.getGUIWidth() / 2, exitBtn.getGUIHeight() / 2);
        }
        private void end()
        {
            while (!UserInfoSemaphore.request())
            {}
            myUserInfo.setInt(1, 0);
            myUserInfo.setInt(2, 0);
            for (int i = 0; i < 5; i++)
                myUserInfo.setString(i, "");
            myUserInfo.store();
            UserInfoSemaphore.release();
            status = STATUS_NULL;
        }
        private void startMyTurn()
        {
            // Start timer
            timeStartedMyTurn = System.currentTimeMillis();
            
            statusLbl.setText("Your turn (" + TIME_FOR_TURN / 1000 + "s before lose turn)");
            
            // Allow me to select position to attack
            eBoard.enableSelecting();
            
            status = STATUS_MY_TURN;
        }
    }
}


/*
 * 1 int for activity
 * 
 * 1 int:
 * (2-bits for status: null(00), requesting(01), in-game-placing(10), in-game-playing(11))
 * (7-bits for my turn number for what info is stored of my move i.e. 0 for have not made my 1st move)
 * (7-bits for board cell (0-99) for where my turn's attack is, value100 means forfeit turn)
 * (3-bits. 1 for read notification. 2 for miss, hit, hit&sunk)
 * 
 * 1 int:
 * (15-bits for opponent id of whom I'm requesting or playing with)
 * (1-bit for initializing chat)
 * (1-bit for message notification)
 * (1-bit for read notification)
 * 
 * 1 string for reveal ship report to loser of a game
 * 
 * 3 strings for messaging
 * 
 * example
 * UserA inc turn# and sends coordinates
 * UserB responds, (miss, hit, hit&sunk). Reset notification
 * UserA receive response, set notification
 * UserB inc turn# and sends coordinates
 * UserA responds, (miss, hit, hit&sunk). Reset notification
 * UserB receive response, set notification
 * 
 */

/*
 * loser gets ship locs, sets chat notification
 * winner gets notification, sets chat notification
 * loser gets notification, done
 */
