import greenfoot.UserInfo;
import java.util.List;
import java.util.HashMap;

/**
 * UserOnlineStatusTracker
 * <p>
 * Listens for activity of other users for determining whether they are online.
 * 
 * @author Taylor Born
 * @version December 2013
 */
public class UserOnlineStatusTracker extends UserInfoThread implements Runnable
{
    private static final int ELAPSE_TIME = 1000;
    private static volatile HashMap<String, OnlinePresence> userOnlinePresence;
    
    public static OnlinePresence getOnlinePresenceFor(String userName)
    {
        return userOnlinePresence.get(userName);
    }

    public UserOnlineStatusTracker()
    {
        userOnlinePresence = new HashMap<String, OnlinePresence>();
        
        load();
    }
    
    private void load()
    {
        while (!UserInfoSemaphore.request())
        {}
        List<UserInfo> list = UserInfo.getTop(Screen.PULL_USERINFO_LIST_SIZE);
        UserInfoSemaphore.release();
        
        for (UserInfo userInfo : list)
        {
            int score = userInfo.getScore();
            
            if (score <= 0)
            {
                list = null;
                break;
            }
            
            String name = userInfo.getUserName();
            
            if (!name.equals(Screen.main.myName))
            {
                OnlinePresence op = userOnlinePresence.get(name);
                int presenceValue = userInfo.getInt(0);
                if (op == null)
                {
                    op = new OnlinePresence(presenceValue);
                    userOnlinePresence.put(name, op);
                    CorrespondentScreen.main.newUser(userInfo);
                }
                else
                {
                    String bits1 = Integer.toBinaryString(userInfo.getInt(1));
                    while (bits1.length() < 32)
                        bits1 = '0' + bits1;
                    String bits2 = Integer.toBinaryString(userInfo.getInt(2));
                    while (bits2.length() < 32)
                        bits2 = '0' + bits2;
                    boolean requestingMe = bits1.substring(0, 2).equals("01") && BitStringUtil.bitStringToNumber(bits2.substring(0, 15)) == Screen.myScore;
                    
                    char ch = (requestingMe ? 'R' : bits1.substring(0, 2).equals("00") ? '-' : 'G');
                    op.update(presenceValue, ch);
                    
                    if (!UserIcons.loadedIconFor(name)) // Won't be reached from constructor since op should be null at that point
                    {
                        while (!UserInfoSemaphore.request())
                        {}
                        UserIcons.commit(name, userInfo);
                        UserInfoSemaphore.release();
                    }
                }
            }
        }
    }
    
    @Override
    public void run()
    {
        while (true)
        {
            load();
            
            do
                ThreadUtil.linger(ELAPSE_TIME);
            while (!Screen.main.running);
            if (end)
                return;
        }
    }
}